import {afterAll, assert, beforeAll, describe, it} from "vitest";
import type {Browser, BrowserContext, Page} from "playwright";
import {chromium} from "playwright";
import {createServer, IncomingMessage, Server, ServerResponse} from "http";
import * as fs from "fs";
import {randomUUID} from "crypto";
import {encode} from "iconv-lite";
import {Transaction} from "tg-mdk-node/dist/tgMdk/Transaction.js";
import {MerchantConfig} from "tg-mdk-node/dist/tgMdk/MerchantConfig.js";
import * as Log4js from "log4js";
import {SearchRequestDto} from "tg-mdk-node/dist/tgMdkDto/Search/SearchRequestDto.js";
import {SearchParameters} from "tg-mdk-node/dist/tgMdkDto/Search/SearchParameters.js";
import {CommonSearchParameter} from "tg-mdk-node/dist/tgMdkDto/Search/CommonSearchParameter.js";
import {SearchResponseDto} from "tg-mdk-node/dist/tgMdkDto/Search/SearchResponseDto.js";
import {CardAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardAuthorizeRequestDto.js";
import {CardAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardAuthorizeResponseDto.js";
import {CardCaptureRequestDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardCaptureRequestDto.js";
import {CardCaptureResponseDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardCaptureResponseDto.js";
import {CardCancelRequestDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardCancelRequestDto.js";
import {CardCancelResponseDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardCancelResponseDto.js";
import {CardReAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardReAuthorizeRequestDto.js";
import {CardReAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/Card/CardReAuthorizeResponseDto.js";
import {MpiAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/Mpi/MpiAuthorizeRequestDto.js";
import {MpiAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/Mpi/MpiAuthorizeResponseDto.js";
import {CvsAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/ConvenienceStore/CvsAuthorizeRequestDto.js";
import {CvsAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/ConvenienceStore/CvsAuthorizeResponseDto.js";
import {CvsCancelRequestDto} from "tg-mdk-node/dist/tgMdkDto/ConvenienceStore/CvsCancelRequestDto.js";
import {CvsCancelResponseDto} from "tg-mdk-node/dist/tgMdkDto/ConvenienceStore/CvsCancelResponseDto.js";
import {FraudDetectionRequestDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetection/FraudDetectionRequestDto.js";
import {FraudDetectionDeviceDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetection/FraudDetectionDeviceDto.js";
import {FraudDetectionHeaderDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetection/FraudDetectionHeaderDto.js";
import {AccountAddRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountAddRequestDto.js";
import {AccountAddResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountAddResponseDto.js";
import {AccountUpdateRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountUpdateRequestDto.js";
import {AccountUpdateResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountUpdateResponseDto.js";
import {CardInfoGetRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoGetRequestDto.js";
import {CardInfoAddRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoAddRequestDto.js";
import {CardInfoAddResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoAddResponseDto.js";
import {CardInfoGetResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoGetResponseDto.js";
import {CardInfoUpdateRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoUpdateRequestDto.js";
import {CardInfoUpdateResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoUpdateResponseDto.js";
import {CardInfoDeleteRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoDeleteRequestDto.js";
import {CardInfoDeleteResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/CardInfoDeleteResponseDto.js";
import {AccountDeleteRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountDeleteRequestDto.js";
import {AccountDeleteResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountDeleteResponseDto.js";
import {AccountRestoreRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountRestoreRequestDto.js";
import {AccountRestoreResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountRestoreResponseDto.js";
import {AccountLinkRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountLinkRequestDto.js";
import {AccountLinkResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountLinkResponseDto.js";
import {AccountCopyRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountCopyRequestDto.js";
import {AccountCopyResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayNowId/AccountCopyResponseDto.js";
import {MpiReAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/Mpi/MpiReAuthorizeRequestDto.js";
import {MpiGetResultRequestDto} from "tg-mdk-node/dist/tgMdkDto/Mpi/MpiGetResultRequestDto.js";
import {MpiGetResultResponseDto} from "tg-mdk-node/dist/tgMdkDto/Mpi/MpiGetResultResponseDto.js";
import {MpiReAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/Mpi/MpiReAuthorizeResponseDto.js";
import {FraudDetectionV2RiskDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2RiskDto.js";
import {FraudDetectionV2RequestDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2RequestDto.js";
import {FraudDetectionV2AirlineDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2AirlineDto.js";
import {FraudDetectionV2PassengerDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2PassengerDto.js";
import {
    FraudDetectionV2BankAccountDto
} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2BankAccountDto.js";
import {
    FraudDetectionV2ParametersDto
} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2ParametersDto.js";
import {FraudDetectionV2CustomerDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2CustomerDto.js";
import {
    FraudDetectionV2BrowserFingerprintDto
} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2BrowserFingerprintDto.js";
import {FraudDetectionV2BillingDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2BillingDto.js";
import {FraudDetectionV2ShippingDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2ShippingDto.js";
import {FraudDetectionV2CorporateDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2CorporateDto.js";
import {FraudDetectionV2CartDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2CartDto.js";
import {FraudDetectionV2ItemDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2ItemDto.js";
import {FraudDetectionV2GiftCardDto} from "tg-mdk-node/dist/tgMdkDto/FraudDetectionV2/FraudDetectionV2GiftCardDto.js";
import {PaypayAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayAuthorizeRequestDto.js";
import {PaypayAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayAuthorizeResponseDto.js";
import {PaypayRefundRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayRefundRequestDto.js";
import {PaypayRefundResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayRefundResponseDto.js";
import {PaypayCancelRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayCancelRequestDto.js";
import {PaypayCancelResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayCancelResponseDto.js";
import {PaypayReAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayReAuthorizeRequestDto.js";
import {PaypayReAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayReAuthorizeResponseDto.js";
import {PaypayTerminateResponseDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayTerminateResponseDto.js";
import {PaypayTerminateRequestDto} from "tg-mdk-node/dist/tgMdkDto/PayPay/PaypayTerminateRequestDto.js";
import {RakutenCancelRequestDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenCancelRequestDto.js";
import {RakutenCancelResponseDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenCancelResponseDto.js";
import {RakutenAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenAuthorizeRequestDto.js";
import {RakutenAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenAuthorizeResponseDto.js";
import {RakutenCaptureRequestDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenCaptureRequestDto.js";
import {RakutenCaptureResponseDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenCaptureResponseDto.js";
import {
    RakutenUpdateAuthorizeRequestDto
} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenUpdateAuthorizeRequestDto.js";
import {
    RakutenUpdateAuthorizeResponseDto
} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenUpdateAuthorizeResponseDto.js";
import {RakutenReAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenReAuthorizeResponseDto.js";
import {RakutenReAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/RakutenPayV2/RakutenReAuthorizeRequestDto.js";
import {AmazonpayAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayAuthorizeResponseDto.js";
import {AmazonpayAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayAuthorizeRequestDto.js";
import {AmazonpayCaptureRequestDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayCaptureRequestDto.js";
import {AmazonpayCaptureResponseDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayCaptureResponseDto.js";
import {AmazonpayCancelRequestDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayCancelRequestDto.js";
import {AmazonpayCancelResponseDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayCancelResponseDto.js";
import {AmazonpayReAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayReAuthorizeRequestDto.js";
import {AmazonpayReAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayReAuthorizeResponseDto.js";
import {AmazonpayRefundRequestDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayRefundRequestDto.js";
import {AmazonpayRefundResponseDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayRefundResponseDto.js";
import {AmazonpayGetAddressRequestDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayGetAddressRequestDto.js";
import {AmazonpayGetAddressResponseDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayGetAddressResponseDto.js";
import {
    AmazonpayUpdateConsentRequestDto
} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayUpdateConsentRequestDto.js";
import {
    AmazonpayUpdateConsentResponseDto
} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayUpdateConsentResponseDto.js";
import {
    AmazonpayUpdateAuthorizeRequestDto
} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayUpdateAuthorizeRequestDto.js";
import {
    AmazonpayUpdateAuthorizeResponseDto
} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayUpdateAuthorizeResponseDto.js";
import {AmazonpayTerminateRequestDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayTerminateRequestDto.js";
import {AmazonpayTerminateResponseDto} from "tg-mdk-node/dist/tgMdkDto/AmazonPay/AmazonpayTerminateResponseDto.js";
import {CarrierAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierAuthorizeRequestDto.js";
import {CarrierAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierAuthorizeResponseDto.js";
import {CarrierCaptureRequestDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierCaptureRequestDto.js";
import {CarrierCaptureResponseDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierCaptureResponseDto.js";
import {CarrierCancelRequestDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierCancelRequestDto.js";
import {CarrierCancelResponseDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierCancelResponseDto.js";
import {CarrierTerminateRequestDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierTerminateRequestDto.js";
import {CarrierTerminateResponseDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierTerminateResponseDto.js";
import {CarrierReAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierReAuthorizeRequestDto.js";
import {CarrierReAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/Carrier/CarrierReAuthorizeResponseDto.js";
import {LinepayAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/LinePay/LinepayAuthorizeRequestDto.js";
import {LinepayAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/LinePay/LinepayAuthorizeResponseDto.js";
import {LinepayCaptureRequestDto} from "tg-mdk-node/dist/tgMdkDto/LinePay/LinepayCaptureRequestDto.js";
import {LinepayCaptureResponseDto} from "tg-mdk-node/dist/tgMdkDto/LinePay/LinepayCaptureResponseDto.js";
import {LinepayCancelRequestDto} from "tg-mdk-node/dist/tgMdkDto/LinePay/LinepayCancelRequestDto.js";
import {LinepayCancelResponseDto} from "tg-mdk-node/dist/tgMdkDto/LinePay/LinepayCancelResponseDto.js";
import {BankAuthorizeRequestDto} from "tg-mdk-node/dist/tgMdkDto/Bank/BankAuthorizeRequestDto";
import {BankAuthorizeResponseDto} from "tg-mdk-node/dist/tgMdkDto/Bank/BankAuthorizeResponseDto";

describe('API Sample', function () {

    function getConfig(): MerchantConfig {
        const ccid = process.env["MERCHANT_CCID"];
        const secret = process.env["MERCHANT_SECRET"];
        const dummy = process.env["DUMMY"] ?? "1";
        if (ccid == undefined || secret == undefined) assert.fail();

        return new MerchantConfig(ccid, secret, dummy);
    }

    function getLogger(): Log4js.Logger {
        Log4js.configure(
            {
                appenders: {system: {type: 'stdout', layout: {type: 'pattern', pattern: '%d %[%5p%] %c %z %m'}}},
                categories: {default: {appenders: ['system'], level: 'info'}}
            }
        );
        return Log4js.getLogger("default");
    }

    describe('Card-Sample', function () {

        let orderId: string;
        let orderId2: string;
        let orderId3: string;
        let accountId: string;
        let cardId: string | undefined;

        it("Card Authorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order1-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CardAuthorizeRequestDto();
            request.amount = "100";
            request.token = await getToken();
            request.orderId = orderId;
            request.withCapture = "false";

            const responseDto = await transaction.execute(request, CardAuthorizeResponseDto);

            assert.equal(responseDto.cardTransactiontype, "a");
            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Card Authorize Test with AccountId", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId2 = 'order2-' + Math.floor(((new Date()).getTime()) / 1000).toString();
            accountId = 'account-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CardAuthorizeRequestDto();
            request.amount = "100";
            request.token = await getToken();
            request.orderId = orderId2;
            request.withCapture = "false";
            request.accountId = accountId;

            const responseDto = await transaction.execute(request, CardAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            assert.notEqual(responseDto?.payNowIdResponse?.account?.cardInfo?.at(0)?.cardId, null);

            cardId = responseDto?.payNowIdResponse?.account?.cardInfo?.at(0)?.cardId;
        });

        it("Card Authorize Test with CardId and Fraud", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId3 = 'order3-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CardAuthorizeRequestDto();
            request.amount = "100";
            request.orderId = orderId3;
            request.withCapture = "false";
            request.accountId = accountId;
            if (cardId == undefined) assert.fail();
            request.cardId = cardId;

            const header = new FraudDetectionHeaderDto("headerName", "headerValue");
            const device = new FraudDetectionDeviceDto("192.168.1.1", [header]);
            request.fraudDetectionRequest = new FraudDetectionRequestDto(device);

            const responseDto = await transaction.execute(request, CardAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            if (responseDto.fraudDetectionResponse != null) {
                assert.equal(responseDto.fraudDetectionResponse.result, "accept");
            }
        });

        it("Order Search Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new SearchRequestDto();
            request.serviceTypeCd = ["card"];
            request.containDummyFlag = "true";
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderId));
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response.orderInfos?.orderInfo?.at(0)?.transactionInfos?.transactionInfo?.at(0)?.properTransactionInfo?.cardTransactionType, "a");
            assert.equal(response.vResultCode, "N001000000000000");
            assert.equal(response.searchCount, "1");
        });

        it("Order Search Test 2", async () => {
            const con = getConfig();
            con.dummyRequest = "0"
            const transaction = new Transaction(getLogger(), con);
            const request = new SearchRequestDto();
            request.serviceTypeCd = ["card"];
            request.containDummyFlag = "true";
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderId));
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response.vResultCode, "N001000000000000");
            assert.equal(response.searchCount, "1");
        });

        it("Card Capture Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardCaptureRequestDto();
            request.orderId = orderId;
            request.amount = "100";
            const response = await transaction.execute(request, CardCaptureResponseDto);
            assert.equal(response.mstatus, "success");
        });

        it("Card Cancel Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardCancelRequestDto();
            request.orderId = orderId;
            request.amount = "100";
            const cancelResponseDto = await transaction.execute(request, CardCancelResponseDto);
            assert.equal(cancelResponseDto.mstatus, "success");
        });

        it("Search Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new SearchRequestDto();
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderId));
            request.serviceTypeCd = ["card"];
            request.containDummyFlag = "true";
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response.vResultCode, "N001000000000000");
            assert.equal(response.searchCount, "1");
            assert.equal(response?.orderInfos?.orderInfo?.length, 1);
            assert.equal(response?.orderInfos?.orderInfo?.at(0)?.transactionInfos?.transactionInfo?.length, 3);
            assert.equal(response?.orderInfos?.orderInfo?.at(0)?.index, "0");
            assert.equal(response?.orderInfos?.orderInfo?.at(0)?.transactionInfos?.transactionInfo?.at(2)?.properTransactionInfo?.cardTransactionType, "vpa");
        });

        it("Card ReAuthorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardReAuthorizeRequestDto();
            request.originalOrderId = orderId;
            request.orderId = orderId + "-2";
            request.amount = "100";
            const response = await transaction.execute(request, CardReAuthorizeResponseDto);
            assert.equal(response.mstatus, "success");
        });

    });

    describe('Mpi-Sample', function () {
        let server: Server<typeof IncomingMessage, typeof ServerResponse>
        beforeAll(() => {
            server = getServer()
        });
        afterAll(() => {
            server.closeAllConnections();
            server.close();
        })
        const redirectionUri = "http://localhost:" + localServerPort.toString() + "/";

        let orderId: string;
        let orderId2: string;

        it("Mpi Authorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new MpiAuthorizeRequestDto();
            request.serviceOptionType = "mpi-complete";
            request.orderId = orderId;
            request.amount = "100";
            request.token = await getToken();
            request.cardholderEmail = 'noreply@veritrans.jp'
            request.withCapture = "false";
            request.redirectionUri = redirectionUri;
            request.verifyResultLink = "1";
            request.httpUserAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/76.0.3800.0 Safari/537.36 Edg/76.0.172.0";
            request.httpAccept = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3";
            request.deviceChannel = "02";

            const responseDto = await transaction.execute(request, MpiAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            if (responseDto.authStartUrl == null) assert.fail();

            await openBrowser();
            await Promise.all([
                    page.waitForURL(redirectionUri),
                    page.goto(responseDto.authStartUrl)
                ]
            );
            closeBrowser();
        });

        it("Mpi GetResult Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            const request = new MpiGetResultRequestDto();
            request.orderId = orderId;

            const responseDto = await transaction.execute(request, MpiGetResultResponseDto);
            assert.equal(responseDto.cardTransactionType, "a");
            assert.equal(responseDto.mstatus, "success");
        });

        it("Mpi ReAuthorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId2 = 'order2-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new MpiReAuthorizeRequestDto();
            request.serviceOptionType = "mpi-complete";
            request.orderId = orderId2;
            request.originalOrderId = orderId;
            request.amount = "100";
            request.withCapture = "false";
            request.redirectionUri = redirectionUri;
            request.verifyResultLink = "1";
            request.httpUserAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/76.0.3800.0 Safari/537.36 Edg/76.0.172.0";
            request.httpAccept = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3";
            request.deviceChannel = "02";

            const responseDto = await transaction.execute(request, MpiReAuthorizeResponseDto);
            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            if (responseDto.authStartUrl == null) assert.fail();

            await openBrowser();
            await Promise.all([
                    page.waitForURL(redirectionUri),
                    page.goto(responseDto.authStartUrl)
                ]
            );
            closeBrowser();
        });

    });

    describe('Cvs-Sample', function () {
        let orderId: string;

        it("Cvs Authorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CvsAuthorizeRequestDto();
            request.serviceOptionType = "sej";
            request.orderId = orderId;
            request.amount = "100";
            request.name1 = "支払い";
            request.name2 = "太郎";
            request.telNo = "090-0000-0000";

            const now = new Date();
            now.setDate(now.getDate() + 5); // + 5 days
            request.payLimit = now.toLocaleDateString('ja-JP', {year: "numeric", month: "2-digit", day: "2-digit",});
            request.paymentType = "0";

            const responseDto = await transaction.execute(request, CvsAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Cvs Cancel Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CvsCancelRequestDto();
            request.serviceOptionType = "sej";
            request.orderId = orderId;
            const response = await transaction.execute(request, CvsCancelResponseDto);
            assert.equal(response.mstatus, "success");
        });

    });

    describe('Bank-Sample', function () {
        let orderId: string;

        it("Bank Authorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new BankAuthorizeRequestDto();
            request.serviceOptionType = "atm";
            request.orderId = orderId;
            request.amount = "100";
            request.name1 = "顧客　氏名";
            request.kana1 = "コキャクシメイカナ";
            request.post1 = "123";
            request.post2 = "4567";
            request.address1 = "東京都港区１－６－１";
            request.telNo = "0123456789";

            const now = new Date();
            request.agreementDate = now.toLocaleDateString('ja-JP', {year: "numeric", month: "2-digit", day: "2-digit",}).replaceAll("/","");
            now.setDate(now.getDate() + 5); // + 5 days
            request.payLimit = now.toLocaleDateString('ja-JP', {year: "numeric", month: "2-digit", day: "2-digit",}).replaceAll("/","");

            request.contents = "表示インフォメーション";
            request.contentsKana = "ヒョウジインフォメーションカナ";

            const responseDto = await transaction.execute(request, BankAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            assert.isNotEmpty(responseDto.shunoKikanNo);
            assert.isNotEmpty(responseDto.customerNo);
            assert.isNotEmpty(responseDto.confirmNo);
        });
    });

    describe('PayNowId-Sample', function () {
        let accountId: string;
        let cardId1: string | undefined;
        let cardId2: string | undefined;

        it("Account Add Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            accountId = 'account-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new AccountAddRequestDto();
            request.accountId = accountId;
            request.token = await getToken();
            request.withAuthorize = "1";
            request.defaultCard = "1";

            const responseDto = await transaction.execute(request, AccountAddResponseDto);

            assert.equal(responseDto?.payNowIdResponse?.account?.accountId, request.accountId);
            assert.equal(responseDto?.mstatus, "success");
            assert.equal(responseDto?.payNowIdResponse?.account?.cardInfo?.at(0)?.cardNumber?.slice(0, 6), process.env["CARDNUMBER_VISA"]?.slice(0, 6));
            assert.equal(responseDto?.payNowIdResponse?.account?.cardInfo?.at(0)?.cardholderName, process.env["CC_NAME"]);
            cardId1 = responseDto?.payNowIdResponse?.account?.cardInfo?.at(0)?.cardId;
        });

        it("Account Update Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AccountUpdateRequestDto();
            request.accountId = accountId;
            const responseDto = await transaction.execute(request, AccountUpdateResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("CardInfo Get Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardInfoGetRequestDto();
            request.accountId = accountId;
            const responseDto = await transaction.execute(request, CardInfoGetResponseDto);

            assert.equal(responseDto?.payNowIdResponse?.account?.cardInfo?.at(0)?.cardId, cardId1);
            assert.equal(responseDto.mstatus, "success");
        });

        it("CardInfo Add Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardInfoAddRequestDto();
            request.accountId = accountId;
            request.token = await getToken("MasterCard");
            request.defaultCard = "0";

            const responseDto = await transaction.execute(request, CardInfoAddResponseDto);

            assert.equal(responseDto.mstatus, "success");
            assert.equal(responseDto?.payNowIdResponse?.account?.cardInfo?.filter(
                value => value?.cardNumber?.slice(0, 6) == process.env["CARDNUMBER_MASTERCARD"]?.slice(0, 6)).length, 1);

            let cardInfo = responseDto?.payNowIdResponse?.account?.cardInfo?.filter(value => value?.cardNumber?.slice(0, 6) == process.env["CARDNUMBER_MASTERCARD"]?.slice(0, 6))[0];
            cardId2 = cardInfo?.cardId;
        });

        it("CardInfo Update Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardInfoUpdateRequestDto();
            request.accountId = accountId;
            if (cardId2 == undefined) assert.fail();
            request.cardId = cardId2;
            request.token = await getToken("JCB");
            request.defaultCard = "0";
            request.withAuthorize = "1";

            const responseDto = await transaction.execute(request, CardInfoUpdateResponseDto);

            assert.equal(responseDto.mstatus, "success");
            assert.equal(responseDto?.payNowIdResponse?.account?.cardInfo?.filter(
                value => value.cardId == cardId2).length, 1);

            let cardInfo = responseDto?.payNowIdResponse?.account?.cardInfo?.filter(value => value.cardId == cardId2)[0];
            assert.equal(cardInfo?.cardNumber?.slice(0, 6), process.env["CARDNUMBER_JCB"]?.slice(0, 6));
        });

        it("CardInfo Delete Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardInfoDeleteRequestDto();
            request.accountId = accountId;
            if (cardId2 == undefined) assert.fail();
            request.cardId = cardId2;
            const responseDto = await transaction.execute(request, CardInfoDeleteResponseDto);
            assert.equal(responseDto.mstatus, "success");

            let request2 = new CardInfoGetRequestDto();
            request2.accountId = accountId;
            let responseDto2 = await transaction.execute(request2, CardInfoGetResponseDto);
            assert.equal(responseDto2?.payNowIdResponse?.account?.cardInfo?.length, 1);
        });

        it("Account Link Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AccountLinkRequestDto();
            request.accountId = accountId;
            request.originalOrderId = "dummyOrderId";
            const responseDto = await transaction.execute(request, AccountLinkResponseDto);
            // assert.equal(responseDto.mstatus, "success");
            assert.equal(responseDto.mstatus, "failure");
        });

        it("Account Delete Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AccountDeleteRequestDto();
            request.accountId = accountId;
            const responseDto = await transaction.execute(request, AccountDeleteResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Account Restore Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AccountRestoreRequestDto();
            request.accountId = accountId;
            const responseDto = await transaction.execute(request, AccountRestoreResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("CardInfo Get Test 2", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CardInfoGetRequestDto();
            request.accountId = accountId;
            const responseDto = await transaction.execute(request, CardInfoGetResponseDto);

            assert.equal(responseDto?.payNowIdResponse?.account?.cardInfo?.at(0)?.cardId, cardId1);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Account Copy Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AccountCopyRequestDto();
            request.accountId = accountId + "_copied";
            request.originalAccountId = accountId;
            request.deleteOriginalAccountId = "1";
            request.addCardInfo = "1";
            request.defaultCardOnly = "1";
            request.cleaningConfig = "1";
            const responseDto = await transaction.execute(request, AccountCopyResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

    });

    describe('FraudV2-Sample', function () {
        let server: Server<typeof IncomingMessage, typeof ServerResponse>
        beforeAll(() => {
            server = getServer()
        });
        afterAll(() => {
            server.closeAllConnections();
            server.close();
        })
        const redirectionUri = "http://localhost:" + localServerPort.toString() + "/";

        let orderId: string;
        let orderId2: string;
        let orderId3: string;
        let orderId4: string;
        let orderId5: string;

        it("Card Authorize with Fraud v2 Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order1-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CardAuthorizeRequestDto();
            request.amount = "100";
            request.token = await getToken();
            request.orderId = orderId;
            request.fraudDetectionV2Request = getFraudObject();

            const responseDto = await transaction.execute(request, CardAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            assert.equal(responseDto.fraudDetectionResponse?.rdResponse?.riskFraudStatusCode, "ACCEPT");
            assert.equal(responseDto.fraudDetectionResponse?.result, "accept");
        });

        it("Card ReAuthorize with Fraud v2 Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId2 = 'order2-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CardReAuthorizeRequestDto();
            request.amount = "100";
            request.orderId = orderId2;
            request.originalOrderId = orderId;
            request.fraudDetectionV2Request = getFraudObject();

            const responseDto = await transaction.execute(request, CardReAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            assert.equal(responseDto.fraudDetectionResponse?.rdResponse?.riskFraudStatusCode, "ACCEPT");
            assert.equal(responseDto.fraudDetectionResponse?.result, "accept");
        });

        it("Card ReAuthorize with NOT fraud detection", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId3 = 'order3-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CardReAuthorizeRequestDto();
            request.amount = "100";
            request.orderId = orderId3;
            request.originalOrderId = orderId2;
            request.withFraudDetection = "false";
            request.fraudDetectionV2Request = getFraudObject();

            const responseDto = await transaction.execute(request, CardReAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");
            assert.equal(responseDto.fraudDetectionResponse, undefined);
        });

        it("Mpi Authorize with Fraud v2 Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId4 = 'order5-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new MpiAuthorizeRequestDto();
            request.serviceOptionType = "mpi-complete";
            request.orderId = orderId4;
            request.amount = "100";
            request.token = await getToken();
            request.redirectionUri = redirectionUri;
            request.verifyResultLink = "1";
            request.httpUserAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/76.0.3800.0 Safari/537.36 Edg/76.0.172.0";
            request.httpAccept = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*!/!*;q=0.8,application/signed-exchange;v=b3";
            request.deviceChannel = "02";
            request.fraudDetectionV2Request = getFraudObject();

            const responseDto = await transaction.execute(request, MpiAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");

            if (responseDto.authStartUrl == null) assert.fail();

            await openBrowser();
            await Promise.all([
                    page.waitForURL(redirectionUri),
                    page.goto(responseDto.authStartUrl)
                ]
            );
            closeBrowser();
        });

        it("Mpi ReAuthorize with Fraud v2 Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId5 = 'order5-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new MpiReAuthorizeRequestDto();
            request.serviceOptionType = "mpi-complete";
            request.orderId = orderId5;
            request.amount = "100";
            request.originalOrderId = orderId4;
            request.redirectionUri = redirectionUri;
            request.verifyResultLink = "1";
            request.httpUserAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/76.0.3800.0 Safari/537.36 Edg/76.0.172.0";
            request.httpAccept = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*!/!*;q=0.8,application/signed-exchange;v=b3";
            request.deviceChannel = "02";
            request.fraudDetectionV2Request = getFraudObject();

            const responseDto = await transaction.execute(request, MpiReAuthorizeResponseDto);

            assert.equal(responseDto.orderId, request.orderId);
            assert.equal(responseDto.mstatus, "success");

            if (responseDto.authStartUrl == null) assert.fail();

            await openBrowser();
            await Promise.all([
                    page.waitForURL(redirectionUri),
                    page.goto(responseDto.authStartUrl)
                ]
            );
            closeBrowser();
        });

        function getFraudObject(): FraudDetectionV2RequestDto {
            let fraud = new FraudDetectionV2RequestDto();
            fraud.fraudDetectionMode = "3"; // dummy response
            fraud.bankAccount = new FraudDetectionV2BankAccountDto();
            fraud.bankAccount.holder = "テスト タロウ";
            fraud.risk = new FraudDetectionV2RiskDto();
            fraud.risk.merchantWebsite = "https://www.example.com";
            fraud.risk.parameters = new FraudDetectionV2ParametersDto();
            fraud.risk.parameters.timeOnFile = "163";
            fraud.customer = new FraudDetectionV2CustomerDto();
            fraud.customer.merchantCustomerId = "USER10001";
            fraud.customer.givenName = "タロウ";
            fraud.customer.surname = "テスト";
            fraud.customer.browserFingerprint = new FraudDetectionV2BrowserFingerprintDto();
            fraud.customer.browserFingerprint.value = "some value";
            fraud.customer.status = "EXISTING";
            fraud.billing = new FraudDetectionV2BillingDto();
            fraud.billing.postcode = "1500022";
            fraud.billing.country = "JP";
            fraud.billing.city = "東京都";
            fraud.billing.street1 = "渋谷区";
            fraud.billing.street2 = "恵比寿南3-5-7";
            fraud.billing.houseNumber1 = "アーバンハウス101";
            fraud.shipping = new FraudDetectionV2ShippingDto();
            fraud.shipping.customer = fraud.customer;
            fraud.shipping.postcode = "1500022";
            fraud.shipping.country = "JP";
            fraud.shipping.city = "東京都";
            fraud.shipping.street1 = "渋谷区";
            fraud.shipping.street2 = "恵比寿南3-5-7";
            fraud.shipping.houseNumber1 = "アーバンハウス101";
            fraud.corporate = new FraudDetectionV2CorporateDto();
            fraud.corporate.name = "DG Financial Technology";
            fraud.cart = new FraudDetectionV2CartDto();
            fraud.cart.items = [new FraudDetectionV2ItemDto()];
            const item = new FraudDetectionV2ItemDto();
            item.quantity = "1";
            item.partNumber = "4912345678904";
            item.productCode = "101";
            item.sku = "10";
            item.merchantItemId = "z12345678";
            item.description = "カレンダー";
            item.originalPrice = "100";
            item.type = "P";
            fraud.cart.items.push(item);
            fraud.giftCard = new FraudDetectionV2GiftCardDto();
            fraud.giftCard.message = "歳末のご挨拶を申し上げます。";
            fraud.airline = new FraudDetectionV2AirlineDto();
            fraud.airline.ticketIssueDate = "2022-06-10";
            fraud.airline.passengers = [new FraudDetectionV2PassengerDto()];
            const passenger = new FraudDetectionV2PassengerDto();
            passenger.email = "foo@example.com";
            passenger.name = "テスト タロウ";
            fraud.airline.passengers.push(passenger)
            return fraud;
        }

    });

    describe('PayPay-Sample', function () {
        let server: Server<typeof IncomingMessage, typeof ServerResponse>
        beforeAll(() => {
            server = getServer()
        });
        afterAll(() => {
            server.closeAllConnections();
            server.close();
        })
        const redirectionUri = "http://localhost:" + localServerPort.toString() + "/";

        let orderId: string;
        let orderId2: string;
        let orderId3: string;
        let successDetailTxnType: string | undefined;

        it("PayPay Authorize Test 1", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new PaypayAuthorizeRequestDto();
            request.orderId = orderId;
            request.serviceOptionType = "online";
            request.accountingType = "0";
            request.amount = "100";
            request.itemId = "payment item";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, PaypayAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            // It *MUST* be returned to the browser in the character code *Shift-JIS*.
            createHtmlFileSJIS(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-paypay/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');
            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        it("PayPay Search Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new SearchRequestDto();
            request.serviceTypeCd = ["paypay"];
            request.containDummyFlag = "true";
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderId));
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response?.vResultCode, "N001000000000000");
            assert.equal(response?.searchCount, "1");
            successDetailTxnType = response?.orderInfos?.orderInfo?.at(0)?.successDetailTxnType;
            assert.notEqual(successDetailTxnType, null);
        });

        it("PayPay Cancel or Refund Test", async () => {
            if (successDetailTxnType === "AuthCapture") {
                const transaction = new Transaction(getLogger(), getConfig());
                const request = new PaypayRefundRequestDto();
                request.orderId = orderId;
                request.serviceOptionType = "online";
                const responseDto = await transaction.execute(request, PaypayRefundResponseDto);
                assert.equal(responseDto.mstatus, "success");
            } else if (successDetailTxnType === "Auth") {
                const transaction = new Transaction(getLogger(), getConfig());
                const request = new PaypayCancelRequestDto();
                request.orderId = orderId;
                request.serviceOptionType = "online";
                const responseDto = await transaction.execute(request, PaypayCancelResponseDto);
                assert.equal(responseDto.mstatus, "success");
            } else {
                console.log(successDetailTxnType);
                assert.fail();
            }
        });

        it("PayPay Authorize Test 2", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId2 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new PaypayAuthorizeRequestDto();
            request.orderId = orderId2;
            request.serviceOptionType = "online";
            request.accountingType = "1";
            request.itemId = "payment item";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, PaypayAuthorizeResponseDto);

            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            // It *MUST* be returned to the browser in the character code *Shift-JIS*.
            createHtmlFileSJIS(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-paypay/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    await page.click('input[type="submit"][value="同意"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        it("PayPay ReAuthorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId3 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new PaypayReAuthorizeRequestDto();
            request.orderId = orderId3;
            request.originalOrderId = orderId2;
            request.serviceOptionType = "online";
            request.amount = "100";

            const responseDto = await transaction.execute(request, PaypayReAuthorizeResponseDto);

            assert.equal(responseDto?.mstatus, "success");
        });

        it("PayPay Terminate Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            const request = new PaypayTerminateRequestDto();
            request.orderId = orderId2;
            request.serviceOptionType = "online";
            request.force = "false";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, PaypayTerminateResponseDto);
            assert.equal(responseDto.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            // It *MUST* be returned to the browser in the character code *Shift-JIS*.
            createHtmlFileSJIS(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-paypay/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    await page.click('input[type="submit"][value="同意"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });
    });

    describe('RakutenPayV2-Sample', function () {
        let server: Server<typeof IncomingMessage, typeof ServerResponse>
        beforeAll(() => {
            server = getServer()
        });
        afterAll(() => {
            server.closeAllConnections();
            server.close();
        })
        const redirectionUri = "http://localhost:" + localServerPort.toString() + "/";

        let orderId: string;
        let orderId2: string;
        let orderId3: string;
        let successDetailTxnType: string | undefined;

        it("RakutenPay Authorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new RakutenAuthorizeRequestDto();
            request.orderId = orderId;
            request.payType = "2";
            request.amount = "100";
            request.withCapture = "false";
            request.successUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, RakutenAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");
            if (responseDto.redirectUrl == null) assert.fail();
            await openBrowser();
            await Promise.all([
                page.waitForURL(new RegExp(".*/dummy-rakuten/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto(responseDto.redirectUrl)
                ]
            );

            await page.click('input[type="submit"]');
            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"]')
                ]
            );
            closeBrowser();
        });

        it("RakutenPay Search Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            const request = new SearchRequestDto();
            request.serviceTypeCd = ["rakuten"];
            request.containDummyFlag = "true";
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderId));
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response?.vResultCode, "N001000000000000");
            assert.equal(response?.searchCount, "1");
            successDetailTxnType = response?.orderInfos?.orderInfo?.at(0)?.successDetailTxnType;
            assert.notEqual(successDetailTxnType, null);
        });

        it("RakutenPay Update Authorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new RakutenUpdateAuthorizeRequestDto();
            request.orderId = orderId;
            request.authExtensionFlag = "true";
            const responseDto = await transaction.execute(request, RakutenUpdateAuthorizeResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("RakutenPay Capture Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new RakutenCaptureRequestDto();
            request.orderId = orderId;
            const responseDto = await transaction.execute(request, RakutenCaptureResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("RakutenPay Cancel Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new RakutenCancelRequestDto();
            request.orderId = orderId;
            const responseDto = await transaction.execute(request, RakutenCancelResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("RakutenPay Authorize Test 2", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId2 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new RakutenAuthorizeRequestDto();
            request.orderId = orderId2;
            request.payType = "2";
            request.accountingType = "1";
            request.itemName = "サンプル商品名";
            request.successUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, RakutenAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");
            if (responseDto.redirectUrl == null) assert.fail();
            await openBrowser();
            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-rakuten/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto(responseDto.redirectUrl)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"]')
                ]
            );

            closeBrowser();
        });

        it("RakutenPay ReAuthorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId3 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new RakutenReAuthorizeRequestDto();
            request.orderId = orderId3;
            request.originalOrderId = orderId2;
            request.amount = "100";
            request.withCapture = "false";
            const response = await transaction.execute(request, RakutenReAuthorizeResponseDto);
            assert.equal(response.mstatus, "success");
        });

    });

    describe('AmazonPay-Sample', function () {
        let server: Server<typeof IncomingMessage, typeof ServerResponse>
        beforeAll(() => {
            server = getServer()
        });
        afterAll(() => {
            server.closeAllConnections();
            server.close();
        })
        const redirectionUri = "http://localhost:" + localServerPort.toString() + "/";

        let orderId: string;
        let orderId2: string;
        let orderId3: string;
        let successDetailTxnType: string | undefined;

        it("Amazon Pay Authorize Test 1", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new AmazonpayAuthorizeRequestDto();
            request.orderId = orderId;
            request.accountingType = "0";
            request.amount = "100";
            request.withCapture = "false";
            request.suppressShippingAddressView = "false";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.authorizePushUrl = "https://example.com/";
            //request.capturePushUrl = "https://example.com/";
            //request.cancelPushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, AmazonpayAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            createHtmlFile(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-amazonpay/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(".*/test-tercerog/.*"), {waitUntil: "domcontentloaded"}),
                    page.click('input[type="submit"][value="続行"]')
                ]
            );

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('button.confirm_btn[type="submit"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        it("Amazon Pay Search Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new SearchRequestDto();
            request.serviceTypeCd = ["amazonpay"];
            request.containDummyFlag = "true";
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderId));
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response?.vResultCode, "N001000000000000");
            assert.equal(response?.searchCount, "1");
            successDetailTxnType = response?.orderInfos?.orderInfo?.at(0)?.successDetailTxnType;
            assert.notEqual(successDetailTxnType, null);
        });

        it("Amazon Pay GetAddress Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AmazonpayGetAddressRequestDto();
            request.orderId = orderId;
            request.addInfoRespFlag = "1";
            const response = await transaction.execute(request, AmazonpayGetAddressResponseDto);
            assert.equal(response?.mstatus, "success");
        });

        it("Amazon Pay UpdateAuthorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AmazonpayUpdateAuthorizeRequestDto();
            request.orderId = orderId;
            request.amount = "200";
            const responseDto = await transaction.execute(request, AmazonpayUpdateAuthorizeResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Amazon Pay Capture Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AmazonpayCaptureRequestDto();
            request.orderId = orderId;
            const responseDto = await transaction.execute(request, AmazonpayCaptureResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Amazon Pay Refund Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AmazonpayRefundRequestDto();
            request.orderId = orderId;
            const responseDto = await transaction.execute(request, AmazonpayRefundResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Amazon Pay Authorize Test 2", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId2 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new AmazonpayAuthorizeRequestDto();
            request.orderId = orderId2;
            request.accountingType = "1";
            request.consentAuthType = "1";
            request.amount = "100";
            request.withCapture = "false";
            request.suppressShippingAddressView = "false";
            request.frequencyUnit = "Month";
            request.frequencyValue = "1";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.authorizePushUrl = "https://example.com/";
            //request.capturePushUrl = "https://example.com/";
            //request.cancelPushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, AmazonpayAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            createHtmlFile(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-amazonpay/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(".*/test-tercerog/.*"), {waitUntil: "domcontentloaded"}),
                    page.click('input[type="submit"][value="続行"]')
                ]
            );

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('button.confirm_btn[type="submit"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        it("Amazon Pay UpdateConsent Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AmazonpayUpdateConsentRequestDto();
            request.orderId = orderId2;
            request.amount = "200";
            const response = await transaction.execute(request, AmazonpayUpdateConsentResponseDto);
            assert.equal(response.mstatus, "success");
        });

        it("Amazon Pay ReAuthorize Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId3 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new AmazonpayReAuthorizeRequestDto();
            request.orderId = orderId3;
            request.originalOrderId = orderId2;
            request.amount = "100";
            request.withCapture = "false";
            const response = await transaction.execute(request, AmazonpayReAuthorizeResponseDto);
            assert.equal(response.mstatus, "success");
        });

        it("Amazon Pay Cancel Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AmazonpayCancelRequestDto();
            request.orderId = orderId3;
            const responseDto = await transaction.execute(request, AmazonpayCancelResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Amazon Pay Terminate Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new AmazonpayTerminateRequestDto();
            request.orderId = orderId2;
            const response = await transaction.execute(request, AmazonpayTerminateResponseDto);
            assert.equal(response.mstatus, "success");
        });

    });

    describe('Carrier-Sample', function () {

        let server: Server<typeof IncomingMessage, typeof ServerResponse>
        beforeAll(() => {
            server = getServer()
        });
        afterAll(() => {
            server.closeAllConnections();
            server.close();
        })
        const redirectionUri = "http://localhost:" + localServerPort.toString() + "/";

        let orderIdDocomo1: string;

        it("Carrier Authorize docomo Test accountingType=0", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderIdDocomo1 = 'order-docomo-1-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CarrierAuthorizeRequestDto();
            request.orderId = orderIdDocomo1;
            request.serviceOptionType = "docomo";
            request.amount = "100";
            request.terminalKind = "1";
            request.itemType = "0";
            request.accountingType = "0";
            request.withCapture = "false";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, CarrierAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            // It *MUST* be returned to the browser in the character code *Shift-JIS*. (DO)
            createHtmlFileSJIS(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-carrier/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            page.click('input[type="submit"][value="次へ"]')

            page.click('input[type="submit"][value="承諾して購入する"]')

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        it("Carrier Capture docomo Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CarrierCaptureRequestDto();
            request.orderId = orderIdDocomo1;
            request.serviceOptionType = "docomo";
            const responseDto = await transaction.execute(request, CarrierCaptureResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("Order Search Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new SearchRequestDto();
            request.serviceTypeCd = ["carrier"];
            request.containDummyFlag = "true";
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderIdDocomo1));
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response.vResultCode, "N001000000000000");
            assert.equal(response.searchCount, "1");
            assert.equal(response.orderInfos?.orderInfo?.at(0)?.properOrderInfo?.crServiceType, "docomo");
        });

        let orderIdDocomo2: string;

        it("Carrier Authorize docomo Test accountingType=4", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderIdDocomo2 = 'order-docomo-2' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CarrierAuthorizeRequestDto();
            request.orderId = orderIdDocomo2;
            request.serviceOptionType = "docomo";
            request.terminalKind = "1";
            request.itemType = "0";
            request.accountingType = "4";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, CarrierAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            // It *MUST* be returned to the browser in the character code *Shift-JIS*. (DO)
            createHtmlFileSJIS(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-carrier/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            page.click('input[type="submit"][value="次へ"]')

            page.click('input[type="submit"][value="承諾して購入する"]')

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        let orderIdDocomo3: string;

        it("Carrier ReAuthorize docomo Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CarrierReAuthorizeRequestDto();
            orderIdDocomo3 = 'order-docomo-3' + Math.floor(((new Date()).getTime()) / 1000).toString();
            request.orderId = orderIdDocomo3;
            request.serviceOptionType = "docomo";
            request.originalOrderId = orderIdDocomo2;
            request.withCapture = "false";
            request.amount = "100";
            const responseDto = await transaction.execute(request, CarrierReAuthorizeResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });


        let orderIdAu: string;

        it("Carrier Authorize au Test accountingType=0", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderIdAu = 'order-au-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CarrierAuthorizeRequestDto();
            request.orderId = orderIdAu;
            request.serviceOptionType = "au";
            request.amount = "100";
            request.terminalKind = "1";
            request.itemType = "0";
            request.accountingType = "0";
            request.withCapture = "false";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, CarrierAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            createHtmlFile(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-carrier/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"][value="購入する"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        it("Carrier Cancel au Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CarrierCancelRequestDto();
            request.orderId = orderIdAu;
            request.serviceOptionType = "au";
            const responseDto = await transaction.execute(request, CarrierCancelResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });


        let orderIdSbKtai: string;

        it("Carrier Authorize sb_ktai Test accountingType=1", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderIdSbKtai = 'order-sb-ktai-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CarrierAuthorizeRequestDto();
            request.orderId = orderIdSbKtai;
            request.serviceOptionType = "sb_ktai";
            request.amount = "100";
            request.terminalKind = "1";
            request.itemType = "0";
            request.accountingType = "1";
            request.d3Flag = "2";

            const now = new Date();
            request.mpFirstDate = now.toLocaleDateString('ja-JP', {
                year: "numeric",
                month: "2-digit",
                day: "2-digit",
            }).replaceAll("/", "");
            request.mpDay = "01";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, CarrierAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            // It *MUST* be returned to the browser in the character code *Shift-JIS*. (SB)
            createHtmlFileSJIS(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-carrier/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"][value="同意"]');
            await page.click('input[type="submit"][value="支払う"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

        it("Carrier Terminate sb_ktai Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new CarrierTerminateRequestDto();
            request.orderId = orderIdSbKtai;
            request.serviceOptionType = "sb_ktai";
            const responseDto = await transaction.execute(request, CarrierTerminateResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });


        let orderIdSbMatomete: string;

        it("Carrier Authorize sb_matomete Test accountingType=1", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderIdSbMatomete = 'order-sb-matomete-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new CarrierAuthorizeRequestDto();
            request.orderId = orderIdSbMatomete;
            request.serviceOptionType = "sb_matomete";
            request.amount = "1050";
            request.terminalKind = "1";
            request.itemType = "0";
            request.accountingType = "1";
            request.itemId = "dummy-sc-100001050";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";

            const responseDto = await transaction.execute(request, CarrierAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.responseContents == null) assert.fail();
            const htmlPath = process.cwd() + "/temp/" + randomUUID().toString() + ".html";
            createHtmlFile(htmlPath, responseDto.responseContents);
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-carrier/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto("file://" + htmlPath)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"][value="購入"]')
                ]
            );

            closeBrowser();
            deleteHtmlFile(htmlPath);
        });

    });

    describe('Linepay-Sample', function () {
        let server: Server<typeof IncomingMessage, typeof ServerResponse>
        beforeAll(() => {
            server = getServer()
        });
        afterAll(() => {
            server.closeAllConnections();
            server.close();
        })
        const redirectionUri = "http://localhost:" + localServerPort.toString() + "/";

        let orderId1: string;
        let orderId2: string;
        let successDetailTxnType: string | undefined;

        it("LINE Pay Authorize Test paymentConfirmType=0", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId1 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new LinepayAuthorizeRequestDto();
            request.orderId = orderId1;
            request.amount = "100";
            request.withCapture = "false";
            request.itemName = "サンプル商品名";
            request.successUrl = redirectionUri;
            request.cancelUrl = redirectionUri;
            request.errorUrl = redirectionUri;
            //request.pushUrl = "https://example.com/";
            request.paymentConfirmType = "0";

            const responseDto = await transaction.execute(request, LinepayAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

            if (responseDto.redirectWebUrl == null) assert.fail();
            await openBrowser();

            await Promise.all([
                    page.waitForURL(new RegExp(".*/dummy-linepay/.*"), {waitUntil: "domcontentloaded"}),
                    page.goto(responseDto.redirectWebUrl)
                ]
            );

            await page.click('input[type="submit"]');

            await Promise.all([
                    page.waitForURL(new RegExp(redirectionUri + ".*")),
                    page.click('input[type="submit"][value="購入"]')
                ]
            );

            closeBrowser();
        });

        it("LINE Pay Search Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            const request = new SearchRequestDto();
            request.serviceTypeCd = ["linepay"];
            request.containDummyFlag = "true";
            request.searchParameters = new SearchParameters(new CommonSearchParameter(orderId1));
            const response = await transaction.execute(request, SearchResponseDto);
            assert.equal(response?.vResultCode, "N001000000000000");
            assert.equal(response?.searchCount, "1");
            successDetailTxnType = response?.orderInfos?.orderInfo?.at(0)?.successDetailTxnType;
            assert.notEqual(successDetailTxnType, null);
        });

        it("LINE Pay Capture Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new LinepayCaptureRequestDto();
            request.orderId = orderId1;
            const responseDto = await transaction.execute(request, LinepayCaptureResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("LINE Pay Cancel Test", async () => {
            const transaction = new Transaction(getLogger(), getConfig());
            const request = new LinepayCancelRequestDto();
            request.orderId = orderId1;
            const responseDto = await transaction.execute(request, LinepayCancelResponseDto);
            assert.equal(responseDto.mstatus, "success");
        });

        it("LINE Pay Authorize Test paymentConfirmType=1", async () => {
            const transaction = new Transaction(getLogger(), getConfig());

            orderId2 = 'order-' + Math.floor(((new Date()).getTime()) / 1000).toString();

            const request = new LinepayAuthorizeRequestDto();
            request.orderId = orderId2;
            request.amount = "100";
            request.withCapture = "false";
            request.itemName = "サンプル商品名";
            //request.pushUrl = "https://example.com/";
            request.paymentConfirmType = "1";
            request.oneTimeKey = "abcdef0123456789abc";

            const responseDto = await transaction.execute(request, LinepayAuthorizeResponseDto);

            assert.equal(responseDto?.orderId, request.orderId);
            assert.equal(responseDto?.mstatus, "success");

        });

    });


    type dummyCardType = "VISA" | "MasterCard" | "JCB";

    // Open the local HTML in Playwright to get the token.
    async function getToken(type: dummyCardType = "VISA"): Promise<string> {
        await openBrowser();
        const htmlPath = process.cwd() + "/resources/token.html";
        await page.goto("file://" + htmlPath);

        await page.fill('#token_api_key', process.env["TOKEN_API_KEY"] ?? "")
        switch (type) {
            case "VISA": {
                await page.fill('#cardnumber', process.env["CARDNUMBER_VISA"] ?? "");
                break;
            }
            case "MasterCard": {
                await page.fill('#cardnumber', process.env["CARDNUMBER_MASTERCARD"] ?? "");
                break;
            }
            case "JCB": {
                await page.fill('#cardnumber', process.env["CARDNUMBER_JCB"] ?? "");
                break;
            }
        }
        await page.fill('#exp-date', process.env["EXP_DATE"] ?? "")
        await page.fill('#cvc', process.env["CVC"] ?? "")
        await page.fill('#ccname', process.env["CC_NAME"] ?? "")

        await Promise.all([
            page.waitForLoadState("networkidle"),
            page.click('#submit')
        ]);

        const response = await page.textContent("#result");
        if (response == null) assert.fail();
        const result = JSON.parse(response);
        if (!result.hasOwnProperty("token")) assert.fail();

        closeBrowser();

        return result.token;
    }

    //region playwright instance
    // #region playwright instance

    let page: Page;
    let browser: Browser;

    async function openBrowser() {
        browser = await chromium.launch({
            headless: false
        });

        const context: BrowserContext = await browser.newContext({
            ignoreHTTPSErrors: false,
            acceptDownloads: true
        });
        page = await context.newPage();
    }

    function closeBrowser() {
        if (!page.isClosed()) {
            browser.close();
        }
    }

    // #endregion
    //endregion

    //region localserver tools
    // #region localserver tools

    const localServerPort = 8125;

    function getServer() {
        return createServer((request, response) => {
            console.log('request ', request.url);
            response.writeHead(200, {'Content-Type': "text/html"});
            response.end("OK", 'utf-8');
        }).listen(localServerPort);
    }

    // #endregion
    //endregion

    //region temporary html file tools
    // #region temporary html file tools

    function createHtmlFile(path: string, content: string) {
        fs.writeFileSync(path, content, {flag: "w", encoding: "utf-8"})
    }

    function createHtmlFileSJIS(path: string, content: string) {
        const buffer = encode(content, "Shift_JIS");
        fs.writeFileSync(path, buffer, {flag: "w", encoding: "utf-8"})
    }

    function deleteHtmlFile(path: string) {
        fs.unlinkSync(path)
    }

    // #endregion
    //endregion

});
