# VeriTrans4G Sample Program for MDK Node.js
本ソフトウェアは VeriTrans4G MDK for Node.js を利用した API利用サンプルプログラムです。

***

## 動作に必要なソフトウェア
本サンプルプログラムの実行に必要なソフトウェアは以下の通りです。あらかじめインストールしておく必要があります。
- [Node.js](https://nodejs.org/ja/)  

本サンプルプログラムの編集を行う場合は、以下のような開発ツールをご利用ください。
- [Microsoft Visual Studio Code](https://code.visualstudio.com/Download)
- [JetBrains WebStorm](https://www.jetbrains.com/ja-jp/webstorm/)


***

## 各種設定について

vitest.config.ts にて以下の環境変数を設定してください。  
マーチャントCCIDおよび認証鍵、Token API Keyはマーチャント管理ポータル(MAP)より確認できます。
#### ApiSample/vitest.config.ts

| 変数名 | 設定内容 |
| --- | --- |
| DUMMY           | テスト用取引(ダミー決済)の場合は "1" を設定する |
| MERCHANT_CCID   | マーチャントCCIDを設定する |
| MERCHANT_SECRET | マーチャント認証鍵を設定する |
| TOKEN_API_KEY   | Token API Keyを設定する(カード決済用) |


## MDKパッケージのインストール

package.json の既定ではMDKの依存関係は定義されていません。
#### ApiSample/package.json
```json
  "dependencies": {
    "iconv-lite": "^0.6.3",
    "log4js": "^6.9.1",
    "playwright": "^1.41.2"
  },
```

そのため、利用するパッケージの形式に合わせ、以下のようなコマンドでnpmパッケージとしてインストールしてください。  
※tg-mdk-node-x.x.x.tgz へのパスの指定は適宜変更してください。

#### 例) ES Modulesの場合 (バージョン2.0.0以降)
```shell
npm install ../mdk/tg-mdk-node-2.0.5.tgz
```

#### 例) CommonJSの場合 (バージョン1.2.0以降～2.0.0未満)
```shell
npm install node-fetch@^2 ../mdk/tg-mdk-node-1.2.5.tgz
```

以下のように依存関係が追加され、package-lock.json が作成されます。
#### ApiSample/package.json
```json
  "dependencies": {
    ...
    "tg-mdk-node": "file:../mdk/tg-mdk-node-2.0.5.tgz"
  },
```


***

## 動作確認

ApiSample/test ディレクトリの api.test.ts に各APIへ要求を行うためのサンプルプログラムが書かれていますので、実装の参考にご利用ください。  
また、 package.json に各API分類ごとのテスト実行スクリプトが定義されています。
```json
  "scripts": {
    "card": "npx vitest -t Card-Sample",
    "mpi": "npx vitest -t Mpi-Sample",
    "cvs": "npx vitest -t Cvs-Sample",
    "bank": "npx vitest -t Bank-Sample",
    "paynowid": "npx vitest -t PayNowId-Sample",
    "fraud": "npx vitest -t FraudV2-Sample",
    "paypay": "npx vitest -t PayPay-Sample",
    "rakutenpay": "npx vitest -t RakutenPayV2-Sample",
    "amazonpay": "npx vitest -t AmazonPay-Sample",
    "carrier": "npx vitest -t Carrier-Sample",
    "linepay": "npx vitest -t Linepay-Sample"
  },
```

本人認証サービスの各APIを呼び出すには、以下のように "mpi" を指定して npm run コマンドを実行してください。  
```shell
npm run mpi
```

場合によっては、Playwrightの実行に失敗するケースがあります。エラーメッセージに従って以下のようにコマンドを実行してください。
```shell
npx playwright install 
```

***

## 改版履歴

- 2024/03 Version 1.1.5 公開
  - 銀行決済に対応
- 2024/03 Version 1.1.4 公開
  - 接続先ドメインを新環境(api3.veritrans.co.jp)に変更
  - 依存パッケージのバージョンを更新
- 2023/11 依存パッケージのバージョンを更新
  - Playwrightについて補足を追記
  - 楽天ペイV2
    - 随時決済に対応
- 2023/08 MPIのサンプルでauthStartUrlを使うよう変更
- 2023/08 キャリア決済およびLINE Payのサンプルを追加
- 2023/07 内容を各APIの実行サンプルに変更
  - ES ModulesおよびCommon JS形式のMDKに対応
- 2022/06 MDKに合わせて依存パッケージのバージョンを更新
  - Azurite Emulatorを利用するよう本ドキュメントを修正
  - Node.jsを16に更新し、Functions ラインタイムバージョン4に対応
- 2021/08 MDKに合わせて依存パッケージのバージョンを更新
- 2021/07 MDKに合わせて依存パッケージのバージョンを更新
- 2021/03 MDKに合わせて依存パッケージのバージョンを更新
- 2020/12 MDKに合わせて依存パッケージのバージョンを更新
- 2020/09 MDKに合わせて依存パッケージのバージョンを更新
- 2020/04 初版公開

***

##### Copyright 2024 DG Financial Technology,Inc.
